/****************************************************************
 * This software is Copyright (C) 1986-1998 by                  *
 *                                                              *
 * F6FBB - Jean-Paul ROUBELAT,  jpr@f6fbb.org                   *
 * 6, rue George Sand                                           *
 * 31120 - Roquettes - France                                   *
 *                                                              *
 * License to copy and use this software is granted for         *
 * non-commercial use provided that it is identified as         *
 *                                                              *
 * "FBB packet-radio BBS software by Jean-Paul ROUBELAT, F6FBB" *
 *                                                              *
 * in all material mentioning or referencing this software      *
 * or this function.                                            *
 *                                                              *
 * These notices must be retained in any copies of any part of  *
 * this documentation and/or software.                          *
 *                                                              *
 * Parts of code have been taken from many other softwares.     *
 * Thanks for the help.                                         *
 ****************************************************************/

#include <serv.h>

char *date (void)
{
	return (__DATE__);
}


char *version (void)
{
	static char buffer[20];

#ifdef BETA
	if (ISGRAPH (LETTRE))
		sprintf (buffer, "%d.%02d%c%d", MAJEUR, MINEUR, LETTRE, BETA);
	else if (BETA)
		sprintf (buffer, "%d.%02d%d", MAJEUR, MINEUR, BETA);
	else
		sprintf (buffer, "%d.%02d", MAJEUR, MINEUR);
#else
#ifdef LETTRE
	if (ISGRAPH (LETTRE))
		sprintf (buffer, "%d.%02d%c", MAJEUR, MINEUR, LETTRE);
	else
#endif
		sprintf (buffer, "%d.%02d", MAJEUR, MINEUR);
#endif
	return (buffer);
}


