/****************************************************************
 * This software is Copyright (C) 1986-1998 by                  *
 *                                                              *
 * F6FBB - Jean-Paul ROUBELAT,  jpr@f6fbb.org                   *
 * 6, rue George Sand                                           *
 * 31120 - Roquettes - France                                   *
 *                                                              *
 * License to copy and use this software is granted for         *
 * non-commercial use provided that it is identified as         *
 *                                                              *
 * "FBB packet-radio BBS software by Jean-Paul ROUBELAT, F6FBB" *
 *                                                              *
 * in all material mentioning or referencing this software      *
 * or this function.                                            *
 *                                                              *
 * These notices must be retained in any copies of any part of  *
 * this documentation and/or software.                          *
 *                                                              *
 * Parts of code have been taken from many other softwares.     *
 * Thanks for the help.                                         *
 ****************************************************************/

#define UK  0
#define RR  1
#define RF  2
#define SI  3
#define HD  4
#define DT  5
#define EF  6
#define ET  7
#define NR  8
#define CN  9
#define CA  10
#define RI  11
#define TX  12
#define TM  13
#define AF  14
#define AT  15
#define SN  16  /* Envoie la suite des datas */

#define NUL 0
#define SOH 1
#define STX 2
#define ETX 3
#define EOT 4
#define ENQ 5
#define ACK 6
#define DLE 16
#define NAK 21
#define CAN 24

#define SA  99  /* start */

#define S   0
#define S1  1
#define SH  2
#define SD  3
#define SE  4
#define ST  5
#define R   6
#define RH  7
#define RD  8
#define AB  9   /* abort */
#define CW  10
#define C   11

#define DP  12  /* Etats transitoires */
#define E1  13
#define E2  14
#define E3  15
#define E4  16
#define E5  17
#define E6  18
#define RP  19  /* Reprise de l'envoi */

